# Generated by Django 2.2.28 on 2023-03-31 06:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0004_auto_20230328_1657'),
    ]

    operations = [
        migrations.AddField(
            model_name='application',
            name='application_css_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='application',
            name='application_default_avatar_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='application',
            name='application_favicon_url',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='application',
            name='application_i18n_force',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='application',
            name='application_i18n_lang',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
