# Generated by Django 2.2.28 on 2023-05-12 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0013_auto_20230512_1608'),
    ]

    operations = [
        migrations.AlterField(
            model_name='application',
            name='api_url',
            field=models.CharField(blank=True, help_text='Without "http(s)://"', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='application',
            name='client_url',
            field=models.CharField(blank=True, help_text='Without "http(s)://"', max_length=255, null=True),
        ),
    ]
