# Generated by Django 2.2.28 on 2023-05-12 13:55

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangoldp_application', '0010_auto_20230331_0921'),
    ]

    operations = [
        migrations.AddField(
            model_name='deployment',
            name='requester',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='deployments', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='deployment',
            name='resolutionDate',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
