# Generated by Django 2.2.28 on 2023-05-12 13:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0011_auto_20230512_1555'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deployment',
            name='status',
            field=models.CharField(blank=True, choices=[('Todo', 'Todo'), ('Doing', 'Doing'), ('Done', 'Done'), ('Failed', 'Failed')], default='Todo', max_length=8, null=True),
        ),
    ]
