# Generated by Django 2.2.28 on 2023-03-28 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0003_application_api_url'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='templaterequiredcomponent',
            name='component',
        ),
        migrations.RemoveField(
            model_name='templaterequiredcomponent',
            name='template',
        ),
        migrations.RemoveField(
            model_name='templaterequiredpackage',
            name='package',
        ),
        migrations.RemoveField(
            model_name='templaterequiredpackage',
            name='template',
        ),
        migrations.RemoveField(
            model_name='application',
            name='template',
        ),
        migrations.AddField(
            model_name='application',
            name='repository',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.DeleteModel(
            name='ApplicationTemplate',
        ),
        migrations.DeleteModel(
            name='TemplateRequiredComponent',
        ),
        migrations.DeleteModel(
            name='TemplateRequiredPackage',
        ),
    ]
