# Generated by Django 2.2.28 on 2023-03-31 06:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0006_applicationservice'),
    ]

    operations = [
        migrations.AlterField(
            model_name='applicationservice',
            name='value',
            field=models.TextField(blank=True, null=True),
        ),
    ]
