# Generated by Django 2.2.28 on 2023-03-31 07:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0009_applicationgraphics'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='application',
            name='application_css_url',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_default_avatar_url',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_favicon_url',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_i18n_force',
        ),
        migrations.RemoveField(
            model_name='application',
            name='application_i18n_lang',
        ),
    ]
