# Generated by Django 2.2.28 on 2023-05-12 14:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0012_auto_20230512_1556'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deployment',
            name='date',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Request date'),
        ),
        migrations.AlterField(
            model_name='deployment',
            name='resolutionDate',
            field=models.DateTimeField(auto_now=True, verbose_name='Resolution date'),
        ),
    ]
