# Generated by Django 2.2.28 on 2023-03-31 07:10

from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_application', '0007_auto_20230331_0857'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationNPM',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('package', models.CharField(blank=True, max_length=255, null=True)),
                ('version', models.CharField(blank=True, max_length=255, null=True)),
                ('application', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='npms', to='djangoldp_application.Application')),
            ],
            options={
                'verbose_name': 'application npm',
                'verbose_name_plural': 'application npms',
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'sib:npm',
                'rdf_context': {'application': 'sib:application', 'package': 'sib:npmpackage', 'version': 'sib:npmversion'},
                'owner_field': 'application__creator',
                'container_path': 'application-npms/',
                'serializer_fields': ['@id', 'package', 'version'],
                'depth': 0,
                'anonymous_perms': ['view'],
                'authenticated_perms': ['inherit'],
                'owner_perms': ['inherit', 'change', 'delete'],
                'superuser_perms': ['inherit'],
            },
        ),
    ]
